@@ -30,7 +30,9 @@ def parse_expression(expression: str, cast: bool = True) -> Tuple:
 
 
 def apply_expression(
-    blocks: Sequence[str], bands: Sequence[Union[str, int]], data: numpy.ndarray,
+    blocks: Sequence[str],
+    bands: Sequence[Union[str, int]],
+    data: numpy.ndarray,
 ) -> numpy.ndarray:
     """Apply rio-tiler expression.
 
@@ -44,10 +46,11 @@ def apply_expression(
         numpy.array: output data.
 
     """
-    data = dict(zip(bands, data))
     return numpy.array(
         [
-            numpy.nan_to_num(numexpr.evaluate(bloc.strip(), local_dict=data))
+            numpy.nan_to_num(
+                numexpr.evaluate(bloc.strip(), local_dict=dict(zip(bands, data)))
+            )
             for bloc in blocks
         ]
     )
