@@ -6,11 +6,12 @@ from typing import Any, Dict, Iterator, Optional, Set, Type, Union
 from urllib.parse import urlparse
 
 import attr
+import httpx
 import pystac
-import requests
 from morecantile import TileMatrixSet
+from rasterio.crs import CRS
 
-from ..constants import WEB_MERCATOR_TMS
+from ..constants import WEB_MERCATOR_TMS, WGS84_CRS
 from ..errors import InvalidAssetName, MissingAssets
 from ..utils import aws_get_object
 from .base import BaseReader, MultiBaseReader
@@ -51,7 +52,7 @@ def fetch(filepath: str, **kwargs: Any) -> Dict:
         return json.loads(aws_get_object(bucket, key, **kwargs))
 
     elif parsed.scheme in ["https", "http", "ftp"]:
-        return requests.get(filepath, **kwargs).json()
+        return httpx.get(filepath, **kwargs).json()
 
     else:
         with open(filepath, "r") as f:
@@ -122,10 +123,11 @@ class STACReader(MultiBaseReader):
     """STAC Reader.
 
     Attributes:
-        filepath (str): STAC Item path, URL or S3 URL.
+        input (str): STAC Item path, URL or S3 URL.
         item (dict or pystac.Item, STAC): Stac Item.
         minzoom (int, optional): Set minzoom for the tiles.
-        minzoom (int, optional): Set maxzoom for the tiles.
+        maxzoom (int, optional): Set maxzoom for the tiles.
+        geographic_crs (rasterio.crs.CRS, optional): CRS to use as geographic coordinate system. Defaults to WGS84.
         include (set of string, optional): Only Include specific assets.
         exclude (set of string, optional): Exclude specific assets.
         include_asset_types (set of string, optional): Only include some assets base on their type.
@@ -153,25 +155,36 @@ class STACReader(MultiBaseReader):
 
     """
 
-    filepath: str = attr.ib()
+    input: str = attr.ib()
     item: pystac.Item = attr.ib(default=None, converter=_to_pystac_item)
+
     tms: TileMatrixSet = attr.ib(default=WEB_MERCATOR_TMS)
     minzoom: int = attr.ib(default=None)
     maxzoom: int = attr.ib(default=None)
+
+    geographic_crs: CRS = attr.ib(default=WGS84_CRS)
+
     include_assets: Optional[Set[str]] = attr.ib(default=None)
     exclude_assets: Optional[Set[str]] = attr.ib(default=None)
+
     include_asset_types: Set[str] = attr.ib(default=DEFAULT_VALID_TYPE)
     exclude_asset_types: Optional[Set[str]] = attr.ib(default=None)
+
     reader: Type[BaseReader] = attr.ib(default=COGReader)
     reader_options: Dict = attr.ib(factory=dict)
+
     fetch_options: Dict = attr.ib(factory=dict)
 
     def __attrs_post_init__(self):
         """Fetch STAC Item and get list of valid assets."""
         self.item = self.item or pystac.Item.from_dict(
-            fetch(self.filepath, **self.fetch_options), self.filepath
+            fetch(self.input, **self.fetch_options), self.input
         )
+
+        # TODO: get bounds/crs using PROJ extension if available
         self.bounds = self.item.bbox
+        self.crs = WGS84_CRS
+
         self.assets = list(
             _get_assets(
                 self.item,
@@ -185,9 +198,11 @@ class STACReader(MultiBaseReader):
             raise MissingAssets("No valid asset found")
 
         if self.minzoom is None:
+            # TODO get minzoom from PROJ extension
             self.minzoom = self.tms.minzoom
 
         if self.maxzoom is None:
+            # TODO get maxzoom from PROJ extension
             self.maxzoom = self.tms.maxzoom
 
     def _get_asset_url(self, asset: str) -> str:
