@@ -1,5 +1,6 @@
 """rio-tiler models."""
 
+import itertools
 import warnings
 from enum import Enum
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
@@ -16,9 +17,9 @@ from rasterio.transform import from_bounds
 from rio_color.operations import parse_operations
 from rio_color.utils import scale_dtype, to_math_type
 
-from .constants import ColorTuple, NumType
 from .errors import InvalidDatatypeWarning
-from .utils import linear_rescale, render
+from .types import ColorMapType, GDALColorMapType, IntervalTuple, NumType
+from .utils import linear_rescale, render, resize_array
 
 
 class NodataTypes(str, Enum):
@@ -48,7 +49,6 @@ class Bounds(RioTilerBaseModel):
 class SpatialInfo(Bounds):
     """Dataset SpatialInfo"""
 
-    center: Tuple[NumType, NumType, int]
     minzoom: int
     maxzoom: int
 
@@ -63,7 +63,7 @@ class Info(SpatialInfo):
     colorinterp: Optional[List[str]]
     scale: Optional[float]
     offset: Optional[float]
-    colormap: Optional[Dict[int, ColorTuple]]
+    colormap: Optional[GDALColorMapType]
 
     class Config:
         """Config for model."""
@@ -72,21 +72,28 @@ class Info(SpatialInfo):
         use_enum_values = True
 
 
-class ImageStatistics(RioTilerBaseModel):
+class BandStatistics(RioTilerBaseModel):
     """Image statistics"""
 
-    percentiles: List[NumType]
-    min: NumType
-    max: NumType
-    std: NumType
+    min: float
+    max: float
+    mean: float
+    count: float
+    sum: float
+    std: float
+    median: float
+    majority: float
+    minority: float
+    unique: float
     histogram: List[List[NumType]]
     valid_percent: float
+    masked_pixels: float
+    valid_pixels: float
 
+    class Config:
+        """Config for model."""
 
-class Metadata(Info):
-    """Dataset metadata and statistics."""
-
-    statistics: Dict[str, ImageStatistics]
+        extra = "allow"  # We allow extra values for `percentiles_{}`
 
 
 def to_coordsbbox(bbox) -> Optional[BoundingBox]:
@@ -105,15 +112,17 @@ class ImageData:
         bounds (BoundingBox, optional): bounding box of the data.
         crs (rasterio.crs.CRS, optional): Coordinates Reference System of the bounds.
         metadata (dict, optional): Additional metadata. Defaults to `{}`.
+        band_names (list, optional): name of each band. Defaults to `["1", "2", "3"]` for 3 bands image.
 
     """
 
     data: numpy.ndarray = attr.ib()
     mask: numpy.ndarray = attr.ib()
-    assets: Optional[List[str]] = attr.ib(default=None)
+    assets: Optional[List] = attr.ib(default=None)
     bounds: Optional[BoundingBox] = attr.ib(default=None, converter=to_coordsbbox)
     crs: Optional[CRS] = attr.ib(default=None)
     metadata: Optional[Dict] = attr.ib(factory=dict)
+    band_names: Optional[List[str]] = attr.ib()
 
     @data.validator
     def _validate_data(self, attribute, value):
@@ -123,6 +132,10 @@ class ImageData:
                 "ImageData data has to be an array in form of (count, height, width)"
             )
 
+    @band_names.default
+    def _default_names(self):
+        return [f"{ix + 1}" for ix in range(self.count)]
+
     @mask.default
     def _default_mask(self):
         return numpy.zeros((self.height, self.width), dtype="uint8") + 255
@@ -140,9 +153,28 @@ class ImageData:
             data (sequence): sequence of ImageData.
 
         """
+        h, w = zip(*[(img.height, img.width) for img in data])
+        if len(set(h)) > 1 or len(set(w)) > 1:
+            warnings.warn(
+                "Cannot concatenate images with different size. Will resize using max width/heigh",
+                UserWarning,
+            )
+            max_h, max_w = max(h), max(w)
+            for img in data:
+                if img.height == max_h and img.width == max_w:
+                    continue
+                img.data = resize_array(img.data, max_h, max_w)
+                img.mask = resize_array(img.mask, max_h, max_w)
+
         arr = numpy.concatenate([img.data for img in data])
         mask = numpy.all([img.mask for img in data], axis=0).astype(numpy.uint8) * 255
-        assets = [img.assets[0] for img in data if img.assets]
+        assets = list(
+            dict.fromkeys(
+                itertools.chain.from_iterable(
+                    [img.assets for img in data if img.assets]
+                )
+            )
+        )
 
         bounds_values = [img.bounds for img in data if img.bounds]
         bounds = bounds_values[0] if bounds_values else None
@@ -150,7 +182,15 @@ class ImageData:
         crs_values = [img.crs for img in data if img.crs]
         crs = crs_values[0] if crs_values else None
 
-        return cls(arr, mask, assets=assets, crs=crs, bounds=bounds)
+        band_names = list(
+            itertools.chain.from_iterable(
+                [img.band_names for img in data if img.band_names]
+            )
+        )
+
+        return cls(
+            arr, mask, assets=assets, crs=crs, bounds=bounds, band_names=band_names
+        )
 
     def as_masked(self) -> numpy.ma.MaskedArray:
         """return a numpy masked array."""
@@ -194,8 +234,8 @@ class ImageData:
         self,
         data: numpy.ndarray,
         mask: numpy.ndarray,
-        in_range: Sequence[Tuple[NumType, NumType]],
-        out_range: Sequence[Tuple[NumType, NumType]] = ((0, 255),),
+        in_range: Sequence[IntervalTuple],
+        out_range: Sequence[IntervalTuple] = ((0, 255),),
         out_dtype: Union[str, numpy.number] = "uint8",
     ):
         """Rescale data."""
@@ -220,7 +260,7 @@ class ImageData:
 
     def post_process(
         self,
-        in_range: Optional[Sequence[Tuple[NumType, NumType]]] = None,
+        in_range: Optional[Sequence[IntervalTuple]] = None,
         out_dtype: Union[str, numpy.number] = "uint8",
         color_formula: Optional[str] = None,
         **kwargs: Any,
@@ -262,12 +302,19 @@ class ImageData:
             metadata=self.metadata,
         )
 
-    def render(self, add_mask: bool = True, img_format: str = "PNG", **kwargs) -> bytes:
+    def render(
+        self,
+        add_mask: bool = True,
+        img_format: str = "PNG",
+        colormap: Optional[ColorMapType] = None,
+        **kwargs,
+    ) -> bytes:
         """Render data to image blob.
 
         Args:
             add_mask (bool, optional): add mask to output image. Defaults to `True`.
             img_format (str, optional): output image format. Defaults to `PNG`.
+            colormap (dict or sequence, optional): RGBA Color Table dictionary or sequence.
             kwargs (optional): keyword arguments to forward to `rio_tiler.utils.render`.
 
         Returns:
@@ -285,32 +332,35 @@ class ImageData:
         data = self.data.copy()
         datatype_range = (dtype_ranges[str(data.dtype)],)
 
-        if img_format in ["PNG"] and data.dtype not in ["uint8", "uint16"]:
-            warnings.warn(
-                f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
-                InvalidDatatypeWarning,
-            )
-            data = self._rescale(data, self.mask, in_range=datatype_range)
+        if not colormap:
+            if img_format in ["PNG"] and data.dtype not in ["uint8", "uint16"]:
+                warnings.warn(
+                    f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
+                    InvalidDatatypeWarning,
+                )
+                data = self._rescale(data, self.mask, in_range=datatype_range)
 
-        elif img_format in ["JPEG", "WEBP"] and data.dtype not in ["uint8"]:
-            warnings.warn(
-                f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
-                InvalidDatatypeWarning,
-            )
-            data = self._rescale(data, self.mask, in_range=datatype_range)
+            elif img_format in ["JPEG", "WEBP"] and data.dtype not in ["uint8"]:
+                warnings.warn(
+                    f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
+                    InvalidDatatypeWarning,
+                )
+                data = self._rescale(data, self.mask, in_range=datatype_range)
 
-        elif img_format in ["JP2OPENJPEG"] and data.dtype not in [
-            "uint8",
-            "int16",
-            "uint16",
-        ]:
-            warnings.warn(
-                f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
-                InvalidDatatypeWarning,
-            )
-            data = self._rescale(data, self.mask, in_range=datatype_range)
+            elif img_format in ["JP2OPENJPEG"] and data.dtype not in [
+                "uint8",
+                "int16",
+                "uint16",
+            ]:
+                warnings.warn(
+                    f"Invalid type: `{data.dtype}` for the `{img_format}` driver. Data will be rescaled using min/max type bounds.",
+                    InvalidDatatypeWarning,
+                )
+                data = self._rescale(data, self.mask, in_range=datatype_range)
 
         if add_mask:
-            return render(data, self.mask, img_format=img_format, **kwargs)
+            return render(
+                data, self.mask, img_format=img_format, colormap=colormap, **kwargs
+            )
 
-        return render(data, img_format=img_format, **kwargs)
+        return render(data, img_format=img_format, colormap=colormap, **kwargs)
