@@ -5,17 +5,18 @@ from typing import Any, Callable, List, Optional, Sequence, Tuple, Type, Union,
 
 from rasterio.crs import CRS
 
-from ..constants import MAX_THREADS, BBox
+from ..constants import MAX_THREADS
 from ..errors import EmptyMosaicError, InvalidMosaicMethod, TileOutsideBounds
 from ..models import ImageData
 from ..tasks import create_tasks, filter_tasks
+from ..types import BBox
 from ..utils import _chunks
 from .methods.base import MosaicMethodBase
 from .methods.defaults import FirstMethod
 
 
 def mosaic_reader(
-    mosaic_assets: Sequence[str],
+    mosaic_assets: Sequence,
     reader: Callable[..., ImageData],
     *args: Any,
     pixel_selection: Union[Type[MosaicMethodBase], MosaicMethodBase] = FirstMethod,
@@ -23,7 +24,7 @@ def mosaic_reader(
     threads: int = MAX_THREADS,
     allowed_exceptions: Tuple = (TileOutsideBounds,),
     **kwargs,
-) -> Tuple[ImageData, List[str]]:
+) -> Tuple[ImageData, List]:
     """Merge multiple assets.
 
     Args:
@@ -71,13 +72,16 @@ def mosaic_reader(
     if not chunk_size:
         chunk_size = threads if threads > 1 else len(mosaic_assets)
 
-    assets_used: List[str] = []
+    assets_used: List = []
     crs: Optional[CRS] = None
     bounds: Optional[BBox] = None
 
     for chunks in _chunks(mosaic_assets, chunk_size):
         tasks = create_tasks(reader, chunks, threads, *args, **kwargs)
-        for img, asset in filter_tasks(tasks, allowed_exceptions=allowed_exceptions,):
+        for img, asset in filter_tasks(
+            tasks,
+            allowed_exceptions=allowed_exceptions,
+        ):
             if isinstance(img, tuple):
                 img = ImageData(*img)
 
