@@ -2,14 +2,16 @@
 
 import os
 from io import BytesIO
-from typing import Any, Dict, Generator, Optional, Sequence, Tuple, Union
+from typing import Any, Dict, Generator, List, Optional, Sequence, Tuple, Union
 
 import numpy
+import rasterio
 from affine import Affine
 from boto3.session import Session as boto3_session
 from rasterio import windows
 from rasterio.crs import CRS
-from rasterio.enums import ColorInterp, MaskFlags
+from rasterio.dtypes import _gdal_typename
+from rasterio.enums import ColorInterp, MaskFlags, Resampling
 from rasterio.features import is_valid_geom
 from rasterio.io import DatasetReader, DatasetWriter, MemoryFile
 from rasterio.rio.helpers import coords
@@ -18,8 +20,9 @@ from rasterio.vrt import WarpedVRT
 from rasterio.warp import calculate_default_transform, transform_geom
 
 from .colormap import apply_cmap
-from .constants import WEB_MERCATOR_CRS, BBox, NumType
+from .constants import WEB_MERCATOR_CRS
 from .errors import RioTilerError
+from .types import BBox, ColorMapType, IntervalTuple
 
 
 def _chunks(my_list: Sequence, chuck_size: int) -> Generator[Sequence, None, None]:
@@ -48,44 +51,128 @@ def aws_get_object(
     return response["Body"].read()
 
 
-def _stats(
-    arr: numpy.ma.array, percentiles: Tuple[float, float] = (2, 98), **kwargs: Any
-) -> Dict:
-    """Calculate array statistics.
+def get_bands_names(
+    indexes: Optional[Sequence[int]] = None,
+    expression: Optional[str] = None,
+    count: Optional[int] = None,
+) -> List[str]:
+    """Define bands names based on expression, indexes or band count."""
+    if expression:
+        return expression.split(",")
+
+    elif indexes:
+        return [str(idx) for idx in indexes]
+
+    elif count:
+        return [str(idx + 1) for idx in range(count)]
+
+    else:
+        raise ValueError(
+            "one of expression or indexes or count must be passed to define band names."
+        )
+
+
+def get_array_statistics(
+    data: numpy.ma.MaskedArray,
+    categorical: bool = False,
+    categories: Optional[List[float]] = None,
+    percentiles: List[int] = [2, 98],
+    **kwargs: Any,
+) -> List[Dict[Any, Any]]:
+    """Calculate per band array statistics.
 
     Args:
-        arr (numpy.ndarray): Input array data to get the stats from.
-        percentiles (tuple, optional): Min/Max percentiles to compute. Defaults to `(2, 98)`.
-        kwargs (optional): Options to forward to numpy.histogram function.
+        data (numpy.ma.MaskedArray): input masked array data to get the statistics from.
+        categorical (bool): treat input data as categorical data. Defaults to False.
+        categories (list of numbers, optional): list of categories to return value for.
+        percentiles (list of numbers, optional): list of percentile values to calculate. Defaults to `[2, 98]`.
+        kwargs (optional): options to forward to `numpy.histogram` function (only applies for non-categorical data).
 
     Returns:
-        dict: numpy array statistics (percentiles, min, max, stdev, histogram, valid_percent).
+        list of dict
 
     Examples:
-        >>> {
-            'percentiles': [38, 147],
-            'min': 20,
-            'max': 180,
-            'std': 28.123562304138662,
-            'histogram': [
-                [1625, 219241, 28344, 15808, 12325, 10687, 8535, 7348, 4656, 1208],
-                [20.0, 36.0, 52.0, 68.0, 84.0, 100.0, 116.0, 132.0, 148.0, 164.0, 180.0]
-            ],
-            'valid_percent': 0.5
-        }
+        >>> data = numpy.ma.zeros((1, 256, 256))
+        >>> get_array_statistics(data)
+        [
+            {
+                'min': 0.0,
+                'max': 0.0,
+                'mean': 0.0,
+                'count': 65536.0,
+                'sum': 0.0,
+                'std': 0.0,
+                'median': 0.0,
+                'majority': 0.0,
+                'minority': 0.0,
+                'unique': 1.0,
+                'percentile_2': 0.0,
+                'percentile_98': 0.0,
+                'histogram': [
+                    [0, 0, 0, 0, 0, 65536, 0, 0, 0, 0],
+                    [-0.5, -0.4, -0.3, -0.19999999999999996, -0.09999999999999998, 0.0, 0.10000000000000009, 0.20000000000000007, 0.30000000000000004, 0.4, 0.5]
+                ],
+                'valid_pixels': 65536.0,
+                'masked_pixels': 0.0,
+                'valid_percent': 100.0
+            }
+        ]
 
     """
-    sample, edges = numpy.histogram(arr[~arr.mask], **kwargs)
-    return dict(
-        percentiles=numpy.percentile(arr[~arr.mask], percentiles)
-        .astype(arr.dtype)
-        .tolist(),
-        min=arr.min().item(),
-        max=arr.max().item(),
-        std=arr.std().item(),
-        histogram=[sample.tolist(), edges.tolist()],
-        valid_percent=((numpy.count_nonzero(~arr.mask)) / float(arr.data.size)) * 100,
-    )
+    if len(data.shape) < 3:
+        data = numpy.expand_dims(data, axis=0)
+
+    output: List[Dict[Any, Any]] = []
+    percentiles_names = [f"percentile_{int(p)}" for p in percentiles]
+
+    for b in range(data.shape[0]):
+        keys, counts = numpy.unique(data[b].compressed(), return_counts=True)
+
+        valid_pixels = float(numpy.ma.count(data[b]))
+        masked_pixels = float(numpy.ma.count_masked(data[b]))
+        valid_percent = round((valid_pixels / data[b].size) * 100, 2)
+        info_px = {
+            "valid_pixels": valid_pixels,
+            "masked_pixels": masked_pixels,
+            "valid_percent": valid_percent,
+        }
+
+        if categorical:
+            out_dict = dict(zip(keys.tolist(), counts.tolist()))
+            h_keys = (
+                numpy.array(categories).astype(keys.dtype) if categories else keys
+            ).tolist()
+            histogram = [
+                [out_dict.get(x, 0) for x in h_keys],
+                h_keys,
+            ]
+        else:
+            h_counts, h_keys = numpy.histogram(data[b][~data[b].mask], **kwargs)
+            histogram = [h_counts.tolist(), h_keys.tolist()]
+
+        percentiles_values = numpy.percentile(
+            data[b].compressed(), percentiles
+        ).tolist()
+
+        output.append(
+            {
+                "min": float(data[b].min()),
+                "max": float(data[b].max()),
+                "mean": float(data[b].mean()),
+                "count": float(data[b].count()),
+                "sum": float(data[b].sum()),
+                "std": float(data[b].std()),
+                "median": float(numpy.ma.median(data[b])),
+                "majority": float(keys[counts.tolist().index(counts.max())].tolist()),
+                "minority": float(keys[counts.tolist().index(counts.min())].tolist()),
+                "unique": float(counts.size),
+                **dict(zip(percentiles_names, percentiles_values)),
+                "histogram": histogram,
+                **info_px,
+            }
+        )
+
+    return output
 
 
 # https://github.com/OSGeo/gdal/blob/b1c9c12ad373e40b955162b45d704070d4ebf7b0/gdal/frmts/ingr/IngrTypes.cpp#L191
@@ -173,7 +260,10 @@ def get_vrt_transform(
         src_dst, bounds, height, width, dst_crs
     ):
         col_off, row_off, w, h = windows.from_bounds(
-            *bounds, transform=src_dst.transform, width=width, height=height,
+            *bounds,
+            transform=src_dst.transform,
+            width=width,
+            height=height,
         ).flatten()
 
         w = windows.Window(
@@ -250,8 +340,8 @@ def non_alpha_indexes(src_dst: Union[DatasetReader, DatasetWriter, WarpedVRT]) -
 
 def linear_rescale(
     image: numpy.ndarray,
-    in_range: Tuple[NumType, NumType],
-    out_range: Tuple[NumType, NumType] = (0, 255),
+    in_range: IntervalTuple,
+    out_range: IntervalTuple = (0, 255),
 ) -> numpy.ndarray:
     """Apply linear rescaling to a numpy array.
 
@@ -305,7 +395,7 @@ def render(
     data: numpy.ndarray,
     mask: Optional[numpy.ndarray] = None,
     img_format: str = "PNG",
-    colormap: Optional[Dict] = None,
+    colormap: Optional[ColorMapType] = None,
     **creation_options: Any,
 ) -> bytes:
     """Translate numpy.ndarray to image bytes.
@@ -314,7 +404,7 @@ def render(
         data (numpy.ndarray): Image array to encode.
         mask (numpy.ndarray, optional): Mask array.
         img_format (str, optional): Image format. See: for the list of supported format by GDAL: https://www.gdal.org/formats_list.html. Defaults to `PNG`.
-        colormap (dict, optional): GDAL RGBA Color Table dictionary.
+        colormap (dict or sequence, optional): RGBA Color Table dictionary or sequence.
         creation_options (optional): Image driver creation options to forward to GDAL.
 
     Returns
@@ -444,7 +534,7 @@ def create_cutline(
 
     Args:
         src_dst (rasterio.io.DatasetReader or rasterio.io.DatasetWriter or rasterio.vrt.WarpedVRT): Rasterio dataset.
-        geometry (dict): GeoJSON feature or GeoJSON geometry. By default the cordinates are considered to be in the dataset CRS. Use `geometry_crs` to set a specific CRS.
+        geometry (dict): GeoJSON feature or GeoJSON geometry. By default the coordinates are considered to be in the dataset CRS. Use `geometry_crs` to set a specific CRS.
         geometry_crs (rasterio.crs.CRS, optional): Input geometry Coordinate Reference System
     Returns:
         str: WKT geometry in form of `POLYGON ((x y, x y, ...)))
@@ -484,3 +574,43 @@ def create_cutline(
         if geom_type == "Polygon"
         else f"MULTIPOLYGON ({str_poly})"
     )
+
+
+def resize_array(
+    data: numpy.ndarray,
+    height: int,
+    width: int,
+    resampling_method: Resampling = "nearest",
+) -> numpy.ndarray:
+    """resize array to a given height and width."""
+    out_shape: Union[Tuple[int, int], Tuple[int, int, int]]
+    if len(data.shape) == 2:
+        count = 1
+        h = data.shape[0]
+        w = data.shape[1]
+        out_shape = (height, width)
+    else:
+        count = data.shape[0]
+        h = data.shape[1]
+        w = data.shape[2]
+        out_shape = (count, height, width)
+
+    # We are using GDAL MEM driver to create a new dataset from the numpy array
+    # ref: https://github.com/rasterio/rasterio/blob/master/rasterio/_io.pyx#L1946-L1955
+    info = {
+        "DATAPOINTER": data.__array_interface__["data"][0],
+        "PIXELS": w,
+        "LINES": h,
+        "BANDS": count,
+        "DATATYPE": _gdal_typename(data.dtype.name),
+    }
+    dataset_options = ",".join(f"{name}={val}" for name, val in info.items())
+    datasetname = f"MEM:::{dataset_options}"
+    with rasterio.open(datasetname, "r+") as src:
+        # if a 2D array is passed, using indexes=1 makes sure we return an 2D array
+        indexes = 1 if len(data.shape) == 2 else None
+        return src.read(
+            out_shape=out_shape,
+            indexes=indexes,
+            resampling=Resampling[resampling_method],
+        )
